from rest_framework import generics, permissions
from .models import NotificationReceipt
from .serializers import NotificationReceiptSerializer


class NotificationReceiptListCreateView(generics.ListAPIView):
    serializer_class = NotificationReceiptSerializer
    permission_classes = [
        permissions.IsAuthenticated
    ]

    def get_queryset(self):
        NotificationReceipt.objects.filter(user=self.request.user)



from django.contrib.auth import get_user_model
from django.test import TestCase
from django.urls import reverse
from push_notifications.models import APNSDevice, GCMDevice
from rest_framework import status
from rest_framework.authtoken.models import Token
from rest_framework.test import APIClient

User = get_user_model()


class DeviceTestCase(TestCase):
    fixtures = ['users']

    def setUp(self):
        super(DeviceTestCase, self).setUp()
        self.client = APIClient()

    def authenticate(self, user=None):
        self.client.credentials()
        if user:
            token, created = Token.objects.get_or_create(user=user)
            self.client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)

    def register_apns_device(self, **kwargs):
        return self.client.post(
            reverse('create_apns_device'),
            format='json',
            **kwargs
        )

    def register_gcm_device(self, **kwargs):
        return self.client.post(
            reverse('create_gcm_device'),
            format='json',
            **kwargs
        )

    @staticmethod
    def get_default_user():
        return User.objects.get(id=1)


class APNSDeviceTestCase(DeviceTestCase):
    def setUp(self):
        super(APNSDeviceTestCase, self).setUp()
        self.user = self.get_default_user()
        self.data = {
            'registration_id': 'F165DC421E885CE7C5F798C9074B5218D2AFAF55F7D7D94A0EDCA4AE3DD7E4A1'
        }

    def test_can_register_device(self):
        self.authenticate(user=self.user)
        count = APNSDevice.objects.count()
        response = self.register_apns_device(data=self.data)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED, response.data)
        self.assertGreater(APNSDevice.objects.count(), count)


class GCMDeviceTestCase(DeviceTestCase):
    def setUp(self):
        super(GCMDeviceTestCase, self).setUp()
        self.user = self.get_default_user()
        self.data = {
            'registration_id': 'F165DC421E885CE7C5F798C9074B5218D2AFAF55F7D7D94A0EDCA4AE3DD7E4A1',
            'cloud_message_type': 'FCM'
        }

    def test_can_register_device(self):
        self.authenticate(user=self.user)
        count = GCMDevice.objects.count()
        response = self.register_gcm_device(data=self.data)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED, response.data)
        self.assertGreater(GCMDevice.objects.count(), count)



from rest_framework import serializers
from .models import NotificationReceipt


class NotificationReceiptSerializer(serializers.ModelSerializer):
    class Meta:
        model = NotificationReceipt
        fields = '__all__'



from django.contrib.auth import get_user_model

User = get_user_model()


class NotificationBase(object):
    _message = 'Hello, World'
    _extra = None
    _badge = None
    _content_available = None
    _thread_id = None

    @property
    def message(self):
        return self._message

    @property
    def additional_data(self):
        data = {
            'extra': self._extra,
            'badge': self._badge,
            'content_available': self._content_available,
            'thread_id': self._thread_id
        }

        return {key: value for key, value in data.items() if value is not None}

    def __str__(self):
        return str({'message': self.message, 'additional_data': self.additional_data})


class FormattedNotification(NotificationBase):
    _data = ()

    @property
    def data(self):
        return self._data

    @property
    def message(self):
        assert isinstance(self.data, tuple)
        return self._message % self.data



from django.contrib.auth import get_user_model
from django.db.models import QuerySet
from django.test import TestCase
from .client import NotificationClient
from .models import NotificationReceipt
from .base import FormattedNotification

User = get_user_model()


class NotificationReceiptTestCase(TestCase):
    fixtures = ['users']

    def setUp(self):
        self.user = User.objects.get(id=1)

    def test_can_create_notification_receipt(self):
        count = NotificationReceipt.objects.count()
        NotificationReceipt.objects.create(user=self.user)
        self.assertGreater(NotificationReceipt.objects.count(), count)


class NotificationClientTestCase(TestCase):
    fixtures = ['users']

    def setUp(self):
        self.users = User.objects.all()
        self.notification = FormattedNotification()
        self.client = NotificationClient()

    def test_send_requires_user_queryset(self):
        try:
            self.client.send_notification(notification=self.notification, users=None)
            self.fail('Send Notification allowed non-queryset argument for users')
        except Exception:
            pass

        try:
            self.client.send_notification(notification=self.notification, users=QuerySet())
            self.fail('Send Notification allowed non-User queryset argument for users')
        except Exception:
            pass

        try:
            self.client.send_notification(notification=self.notification, users=self.users)
        except Exception as e:
            self.fail('Notification Failed to Send: %s' % e)

    def test_send_require_notification_class(self):
        try:
            self.client.send_notification(notification=None, users=self.users)
            self.fail('Send Notification allowed None notification object')
        except Exception:
            pass

        try:
            self.client.send_notification(notification=self.notification, users=self.users)
        except Exception as e:
            self.fail('Notification Failed to Send: %s' % e)

    def test_send_creates_receipt(self):
        count = NotificationReceipt.objects.count()
        self.client.send_notification(notification=self.notification, users=self.users)
        self.assertGreater(NotificationReceipt.objects.count(), count)



from django.contrib import admin
from .models import NotificationReceipt


class NotificationReceiptAdmin(admin.ModelAdmin):
    list_display = ('user', 'payload', 'timestamp')
admin.site.register(NotificationReceipt, NotificationReceiptAdmin)



from setuptools import setup
setup(
  name='lp_push_notifications',
  packages=['lp_push_notifications'],
  package_data={'lp_push_notifications': ['migrations/*']},
  version='0.1.6',
  description='REST Framework for Push Notifications',
  author='Jim Simon',
  author_email='hello@launchpeer.com',
  url='https://github.com/Launchpeer/django-rest-push-notifications',
  download_url='https://github.com/Launchpeer/django-rest-push-notifications/archive/master.tar.gz',
  keywords=[],
  classifiers=[],
)



from django.contrib.auth import get_user_model
from django.db import models

User = get_user_model()


class NotificationReceipt(models.Model):
    user = models.ForeignKey(User)
    timestamp = models.DateTimeField(auto_now_add=True)
    has_read = models.BooleanField(default=False)
    payload = models.TextField(default='')
